varying lowp vec4 v_color;
varying lowp vec2 v_texCoord;
varying mediump vec2 v_uvOffset;

uniform sampler2D u_texture;
uniform lowp vec4 u_materialDiffuse;

/*
x = X direction offset
y = Y-direction offset
z = Alpha power (remove alpha < 1.0)
*/
uniform lowp vec4 Settings;

void main() {
	// Put uniforms into variables for better readabilty
	lowp float alphaPower = max(Settings.z, 1.0);
	
	// Sample textures
	lowp vec4 color = texture2D(u_texture, v_texCoord);
	lowp float offsetAlpha = texture2D(u_texture, v_uvOffset).a;

	// Drop shaodw power
	lowp float shadowAlpha = pow(offsetAlpha, alphaPower) * u_materialDiffuse.a;
	lowp float baseAlpha = clamp(shadowAlpha + color.a, 0.0, 1.0);
	
	// Compose
	lowp float val = clamp(shadowAlpha * (1.0-color.a), 0.0, 1.0);
	lowp vec3 finalColor = mix(color.rgb, u_materialDiffuse.rgb, val);
	
	gl_FragColor = vec4(finalColor, baseAlpha);
}
