uniform lowp sampler2D u_texture; // Mask
varying lowp vec2 v_texCoord;
varying lowp float v_texCoordY;
varying lowp vec4 v_color;

varying mediump vec3 v_normal;
varying mediump vec3 v_camDir;
varying mediump vec3 v_lightDir;

uniform lowp vec4 u_color4;
uniform lowp vec4 u_materialAmbient;
uniform lowp vec4 u_materialDiffuse;
//uniform lowp vec4 u_materialEmissive;
uniform lowp vec4 u_materialSpecular;
uniform lowp float u_materialShininess;

//uniform mediump vec4 texture_settings; // x = diffuse tiling, y = normal tiling, z = mask tiling, w = normal strength
uniform lowp vec4 lighting_settings; // x = diffuse wrap, y = shadow strength, z = unused, w = unused

uniform lowp vec4 papercolor1; //default vec4(1.0, 0.949, 0.874, 1.0);
uniform lowp vec4 papercolor2; //default vec4(0.996, 0.827, 0.545, 1.0);

void main() {
	// * VERT INPUTS *
	mediump vec3 L = normalize(v_lightDir);
	mediump vec3 CD = normalize(v_camDir);
	mediump vec3 N = normalize(v_normal);

	// * TEXTURE SAMPLERS *
	lowp vec3 colorMap = vec3(1.0, 1.0, 1.0);
	//lowp float maskMap = texture2D(u_texture2, v_texCoord * texture_settings.z).b;
	lowp float maskMap = texture2D(u_texture, v_texCoord).b;

	// * ALBEDO * 
	lowp vec3 ret = vec3(0.0, 0.0, 0.0);

	// Border color, a special color along the vtx painted mesh border, broken upp with a tiling mask
	lowp vec3 maskedColor = mix(vec3(1.0, 1.0, 1.0), v_color.rgb, maskMap);
	lowp vec3 borderColor = u_materialDiffuse.rgb;
	lowp float boost = pow(v_color.r, 0.8) + 0.1;
	lowp vec3 albedo = mix( borderColor, u_color4.rgb, maskedColor * boost );
	albedo = colorMap * albedo;

	lowp vec3 papercolor = mix( papercolor1.rgb, papercolor2.rgb, v_texCoordY ); 
	albedo = mix( papercolor, albedo, v_color.a );

	// * DIFFUSE * 
	lowp float diffuse = dot(L, N);
	// Diffuse Wrap
	diffuse = ( diffuse + lighting_settings.x ) / ( 1.0 + lighting_settings.x );

	// Shadow Strength
	diffuse = clamp( diffuse, lighting_settings.y, 1.0 );

	lowp vec3 diffuseColor = albedo * (diffuse + u_materialAmbient.rgb);

	// * SPECULAR *
	// NOTE: current hardcoded lightparams should come from the actual lightsource
	//mediump float specularGlossiness = u_materialShininess;
	//mediump vec4 lightColor = vec4(1.0, 1.0, 1.0, 1.0);
	lowp vec3 specularColor = u_materialSpecular.rgb;
	mediump vec3 H = normalize(L + CD);
	mediump float NdotH = clamp(dot(N, H), 0.0, 1.0);
	specularColor = specularColor * pow(NdotH, u_materialShininess);

	// * FINAL COMP *
	ret = ret + specularColor;
	ret = ret + diffuseColor;

	gl_FragColor = vec4(ret, 1.0);
}
