// INPUTS
uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp mat4 u_modelMatrix;
uniform highp mat4 u_viewMatrix;

attribute highp vec4 a_position;
attribute mediump vec4 a_color;
attribute lowp vec2 a_texCoord;

// OUTPUTS
attribute mediump vec3 a_normal;
varying mediump vec3 v_normal;
varying lowp vec2 v_texCoord;
varying lowp float v_texCoordY;
varying lowp vec4 v_color;
varying mediump vec3 v_camDir;
varying mediump vec3 v_lightDir;

uniform mediump vec4 texture_settings; // x = diffuse tiling, y = normal tiling, z = mask tiling, w = normal strength

void main() {
	// * POSITIONS *
	gl_Position = u_modelViewProjectionMatrix * a_position;

	// * NORMALS *
	v_normal = normalize((u_modelMatrix * vec4(a_normal, 0.0)).xyz);

	const vec3 u_materialEmissive = vec3(0.27, 0.84, 0.73);
	v_lightDir = normalize((vec4(u_materialEmissive.xyz * 2.0 - 1.0, 0.0)).xyz);

	v_camDir = vec3(u_viewMatrix[0][2], u_viewMatrix[1][2], u_viewMatrix[2][2]);

	// * COLOR * 
	v_color = a_color;

	// * TEX COORDINATES *
	v_texCoord = a_texCoord * texture_settings.z;
	v_texCoordY = a_texCoord.y;
}
