uniform lowp sampler2D u_texture;

varying mediump vec2 v_texCoord;
varying mediump vec2 v_texCoord2;

varying mediump vec3 v_normal;
varying mediump vec3 v_camDir;
varying mediump vec3 v_lightDir;

void main() {
	// * CONSTANTS *
	const lowp vec3 u_color4 = vec3(0.23, 0.81, 0.93);
	const lowp vec3 u_materialAmbient = vec3(0.23, 0.55, 0.69);
	const lowp vec3 u_materialDiffuse = vec3(1.0, 1.0, 1.0);
	const lowp vec3 u_materialSpecular = vec3(0.15, 0.15, 0.15);
	const lowp float u_materialShininess = 0.35;

	// * VERT INPUTS *
	mediump vec3 L = normalize(v_lightDir);
	mediump vec3 CD = normalize(v_camDir);
	mediump vec3 N = normalize(v_normal);

	// * "Fresnel" TERM *
	lowp float F = clamp(1.0 - pow(dot(N, CD) * 1.5, 0.5), 0.0, 1.0);

	// * TEXTURE SAMPLERS *
	lowp float stripe = texture2D(u_texture, v_texCoord).g;
	lowp float depth = texture2D(u_texture, v_texCoord2).a;
	
	lowp vec3 colorMap = mix(u_materialAmbient, u_materialDiffuse * 0.95, depth);
	colorMap = mix(colorMap, u_materialDiffuse, stripe);

	// * ALBEDO * 
	lowp vec3 ret = vec3(0.0, 0.0, 0.0);

	// Border color, a special color along the vtx painted mesh border, broken upp with a tiling mask
	lowp vec3 albedo = colorMap * u_color4;

	// * DIFFUSE * 
	lowp float diffuse = dot(L, N);
	diffuse = ( diffuse + 10.0 ) / 11.0;

	// Shadow Strength
	diffuse = clamp( diffuse, 1.3, 1.0 );

	lowp vec3 diffuseColor = albedo * diffuse;

	// * SPECULAR *
	// NOTE: current hardcoded lightparams should come from the actual lightsource
	lowp vec3 specularColor = u_materialSpecular;
	mediump vec3 H = normalize(L + CD);
	mediump float NdotH = clamp(dot(N, H), 0.0, 1.0);
	specularColor = specularColor * pow(NdotH, u_materialShininess);

	// * FINAL COMP *
	ret = ret + specularColor;
	ret = ret + diffuseColor;

	gl_FragColor = vec4(ret + F, 1.0);
}
