uniform highp mat4 u_modelMatrix;
uniform highp mat4 u_modelViewProjectionMatrix;
// For camera information
uniform highp mat4 u_viewMatrixInverse;
// For transforming from world space to local space
// So we can calculate lighting in tangent space
uniform highp mat4 u_modelMatrixInverseTranspose;
// Lighting Matrix
uniform highp mat4 u_directionalLightMatrix;

// Attributes Inputs
attribute highp vec4 a_position;
attribute mediump vec3 a_normal;
attribute mediump vec3 a_tangent;
attribute mediump vec3 a_binormal;
attribute lowp vec2 a_texCoord;

// Varying Outputs
varying mediump vec3 v_worldPos;
varying mediump vec3 v_normal;
varying mediump vec3 v_tangent;
varying mediump vec3 v_binormal;

varying lowp vec2 v_texCoord;
varying mediump vec3 v_camPos;
varying mediump vec3 v_lightDir;

void main() {
	// Transform local space position
	gl_Position = u_modelViewProjectionMatrix * a_position;
	
	v_worldPos = (u_modelMatrix * a_position).xyz;
	
	v_normal = normalize((u_modelMatrixInverseTranspose * vec4(a_normal, 0.0))).xyz;
	v_tangent = normalize((u_modelMatrixInverseTranspose * vec4(a_tangent, 0.0))).xyz;
	v_binormal = normalize((u_modelMatrixInverseTranspose * vec4(a_binormal, 0.0))).xyz;
	
	// Light and Camera
	v_lightDir = normalize(-u_directionalLightMatrix[2].xyz);
	v_camPos = vec3(u_viewMatrixInverse[3][0], u_viewMatrixInverse[3][1], u_viewMatrixInverse[3][2]);

	// Pass attributes as is
	v_texCoord = a_texCoord;
}
