// Map 128 to PI (180 degrees, Using the Shininess parameter)
#define MAP_TO_PI 1.0 * 3.1415926 * 0.0078125
#define SCALE 0.001

uniform lowp mat4 u_modelViewProjectionMatrix;

uniform lowp float u_materialShininess;
uniform lowp vec4 u_materialAmbient;

attribute lowp vec4 a_position;
attribute lowp vec4 a_color;
attribute lowp vec2 a_texCoord;

varying lowp vec4 v_color;
varying lowp vec2 v_texCoord;
varying lowp float v_rotatedPosition;
varying lowp float v_size;

void main() {
	gl_Position = u_modelViewProjectionMatrix * a_position;
	v_color = a_color;
	v_texCoord = a_texCoord;

	// Rotate the object-space position as they are always the same value
	float alpha = u_materialShininess * MAP_TO_PI;
	float cU = a_position.x * SCALE;
	float cV = a_position.y * SCALE;
	
	v_rotatedPosition = cU * cos(alpha) - cV * sin(alpha);
	v_size = u_materialAmbient.r * 1024.0;
}