// Textures
uniform lowp sampler2D u_texture; // Diffuse Texture
uniform lowp sampler2D u_texture1; // Normalmap Texture

// Varying Inputs
varying lowp vec2 v_texCoord;
varying mediump vec3 v_lightDir;
varying mediump vec3 v_halfDir;

// Engine Uniforms
uniform lowp vec4 u_directionalLightColor;
uniform lowp vec4 u_materialAmbient;
uniform lowp vec4 u_materialDiffuse;
uniform lowp vec4 u_materialSpecular;
uniform lowp float u_materialShininess;

// Custom Uniforms
// x = X Tile Factor
// y = Y Tile Factor
// z = Normalmap Strength
// w = Specular Strength
uniform mediump vec4 Settings;

void main() {
	// Tile texcoords (does not work with atlases!)
	mediump vec2 tiled = v_texCoord;
	tiled.x *= Settings.x;
	tiled.y *= Settings.y;

	// Sample diffuse texture
	lowp vec4 diffuseMap = texture2D(u_texture, tiled);
	// Sample normal texture
	lowp vec3 normalMap = texture2D(u_texture1, tiled).rgb;
	normalMap = mix(vec3(0.5, 0.5, 1.0), normalMap, clamp(Settings.z, 0.0, 1.0));
	normalMap = normalMap * 2.0 - 1.0;
	//normalMap.y *= -1.0;
	
	// Colors
	lowp vec4 matAmbient = u_materialAmbient;
	lowp vec4 matDiffuse = u_materialDiffuse;
	lowp vec4 matSpecular = u_materialSpecular;
	lowp vec4 lightColor = u_directionalLightColor;
	//lowp vec4 lightColor = vec4(1.0, 0.5, 0.5, 1.0);
	
	// Gamma correct colors (Use 2.0 to use sqrt instead of pow)
	diffuseMap.rgb = diffuseMap.rgb * diffuseMap.rgb;

	// Normal (normalmap)
	mediump vec3 N = normalize(normalMap);
	mediump vec3 L = normalize(v_lightDir);
	mediump vec3 H = normalize(v_halfDir);

	// Calcuate lighting factors
	mediump float NdotL = max(dot(N, L), 0.0);
	mediump float NdotH = max(dot(N, H), 0.0);
	
	//NdotL = (NdotL + 1.0) / 2.0;
	//NdotL = clamp(NdotL, 0.0, 1.0);

	// Diffuse
	lowp vec3 diffuseColor = lightColor.rgb *
				 matDiffuse.rgb *
				 diffuseMap.rgb *
				 NdotL;

	// Specular
	mediump float gloss = max(u_materialShininess * 8.0, 1.0);
	mediump float spec = max(pow(NdotH, gloss) * Settings.w, 0.0);
	lowp vec3 specularColor = matSpecular.rgb * spec;

	// Composed
	lowp vec3 finalColor = matAmbient.rgb;
	finalColor = finalColor + diffuseColor;
	finalColor = finalColor + specularColor;
	
	finalColor = sqrt(finalColor);

	gl_FragColor = vec4(finalColor, diffuseMap.a + spec);
}
