uniform highp mat4 u_modelViewProjectionMatrix;
// For camera information
uniform highp mat4 u_viewMatrixInverse;
// For transforming from world space to local space
// So we can calculate lighting in tangent space
uniform highp mat4 u_modelMatrixInverse;
// Lighting Matrix
uniform highp mat4 u_directionalLightMatrix;

// Attributes Inputs
attribute highp vec4 a_position;
attribute mediump vec3 a_normal;
attribute mediump vec3 a_tangent;
attribute mediump vec3 a_binormal;
attribute lowp vec2 a_texCoord;
attribute lowp vec4 a_color;

// Varying Outputs
varying lowp vec2 v_texCoord;
varying mediump vec3 v_lightDir;
varying mediump vec3 v_halfDir;

void main() {
	// Transform local space position
	gl_Position = u_modelViewProjectionMatrix * a_position;

	// Create transposed TBN matrix for transforming from local space to tangent space
	mediump mat3 TBN = mat3(
		a_tangent.x, a_binormal.x, a_normal.x,
		a_tangent.y, a_binormal.y, a_normal.y,
		a_tangent.z, a_binormal.z, a_normal.z
	);
	
	// Light and Camera
	vec3 lightDir = normalize(-u_directionalLightMatrix[2].xyz);
	vec3 camPos = vec3(u_viewMatrixInverse[3][0], u_viewMatrixInverse[3][1], u_viewMatrixInverse[3][2]);

	// Local space Light and Camera
	vec3 localLightDir = (u_modelMatrixInverse * vec4(lightDir, 0.0)).xyz;
	vec3 localCamPos = (u_modelMatrixInverse * vec4(camPos, 1.0)).xyz;
	vec3 localViewDir = localCamPos - a_position.xyz;
	
	// Transform light and view to tangent space
	v_lightDir = normalize(TBN * localLightDir);
	vec3 viewDir = normalize(TBN * localViewDir);
	v_halfDir = normalize(v_lightDir + viewDir);

	// Pass attributes as is
	v_texCoord = a_texCoord;
}
