varying lowp vec2 v_texCoord;

uniform sampler2D u_texture;
uniform ivec2 u_textureSize;
	
uniform lowp vec4 u_materialAmbient;
uniform lowp vec4 u_materialDiffuse;
uniform lowp vec4 u_materialSpecular;
uniform lowp vec4 u_materialEmissive;

varying lowp vec4 v_color;
uniform lowp vec4 Thickness;

void main() {
	///////////// Base Color /////////////////
	
	vec4 color = texture2D(u_texture, v_texCoord);
	
	vec3 ambient = color.r * u_materialAmbient.rgb * (1.0 - color.g);
	vec3 diffuse = color.g * u_materialDiffuse.rgb * u_materialDiffuse.a;
	vec3 specular = color.b * u_materialSpecular.rgb * u_materialSpecular.a;

	///////////// Outline /////////////////

	lowp float step_size_h = Thickness.x / (float(u_textureSize.x) * 100.0);
	lowp float step_size_v = Thickness.x / (float(u_textureSize.y) * 100.0);
	
	// Get delta between current and last pixel horisontally
	lowp float alpha_h = 0.0;
	alpha_h += texture2D(u_texture, v_texCoord + vec2(-step_size_h, 0.0)).a;
	alpha_h -= texture2D(u_texture, v_texCoord + vec2(+step_size_h, 0.0)).a;
	
	// Get delta between current and next pixel vertically
	lowp float alpha_v = 0.0;
	alpha_v += texture2D(u_texture, v_texCoord + vec2(0.0, -step_size_v)).a;
	alpha_v -= texture2D(u_texture, v_texCoord + vec2(0.0, +step_size_v)).a;
	
	lowp float alpha = clamp(abs(alpha_h) + abs(alpha_v), 0.0, 1.0);
	alpha = (1.0 - color.a) * alpha;

	// Mix base color with outline color
	vec4 final = vec4(ambient + diffuse + specular, color.a) * v_color;
	gl_FragColor = mix(final, u_materialEmissive, alpha);
}
